import re

from ey_commerce_lib.dxm.schemas.order import DxmOrderRule, DxmOrderRuleCond, DxmOrderReviewRule, DxmOrderLogisticsRule
from ey_commerce_lib.utils.dxm import get_remaining_ship_time, get_data_custom_mark_to_str
from ey_commerce_lib.utils.list_util import get_str_list_first_not_blank_or_none
from lxml import html


def get_single_order_item_list(order_element: html.HtmlElement):
    """
    获取单个订单项的订单列表
    :return:
    """
    sku_list = list()
    sku_element_list = order_element.xpath('.//tr[contains(@class, "pairProInfo")]')
    for sku_element in sku_element_list:
        sku = sku_element.xpath('.//a[contains(@class, "pairProInfoSku")]/text()')[0]
        img = get_str_list_first_not_blank_or_none(sku_element.xpath('.//img/@data-original'))
        quantity = sku_element.xpath('.//span[@class="limingcentUrlpicson"]/following-sibling::span[1]/text()')[0]
        # 价格
        currency, price = \
            sku_element.xpath('.//span[@class="limingcentUrlpicson"]/parent::*/following-sibling::p[1]/text()')[
                0].split(' ')
        # 变种
        variants = sku_element.xpath(
            './/p[@class="pairProInfoName orderPairNameIsOverlength"]/span[@class="isOverLengThHide"]/text()')
        variant_list = list()
        for variant in variants:
            variant_list.append({
                'name': variant.split('：')[0].strip(),
                'value': variant.split('：')[1].strip()
            })
        # 获取来源数据
        source_presentation_element_list = (sku_element.
                                            xpath('.//ul[@id="dropSourceUrl"]/li[@role="presentation"]/a/text()'))
        source_presentation_element_list = [source_presentation_element.strip().split('：')[1]
                                            for source_presentation_element in source_presentation_element_list]
        sku_list.append({
            'sku': sku,
            'quantity': int(quantity),
            'price': float(price.replace(',', '')),
            'currency': currency,
            'img': img,
            'variants': variant_list,
            'source_list': source_presentation_element_list
        })
    # 订单金额
    currency, price = order_element.xpath('./td[2]/text()')[0].strip().split(' ')
    recipient = None
    country = None
    if order_element.xpath('.//td[3]/span/text()')[0] != '「」':
        # 收件人
        recipient = order_element.xpath('.//td[3]/span/text()')[0]
        country = order_element.xpath('.//td[3]/span/text()')[1].replace('「', '').replace('」', '')
    # 订单号
    order_id = order_element.xpath('.//td[4]//a[contains(@class, "limingcentUrlpic")]/text()')[0]
    # 备注
    remark = get_str_list_first_not_blank_or_none(
        order_element.xpath('.//td[4]//span[contains(@class, "buyerNotes")]/@data-content'))
    # 下单时间等信息
    time_element_list = order_element.xpath('.//td[5]/div')
    time_dict = dict()
    for time_element in time_element_list:
        # 如果存在id,说明是剩余发货时间需要动态计算
        if len(time_element.xpath('@id')) > 0:
            script_str = time_element.xpath('./script/text()')[0].strip()
            m = re.search(r'addTimer\(".*?",\s*(\d+),', script_str)
            remaining_ship_time = None
            if m:
                # 剩余发货时间
                remaining_ship_time = get_remaining_ship_time(int(m.group(1)))
            # 如果是None就代表已到期
            time_dict['remaining_ship_time'] = remaining_ship_time

        else:
            time_name, time_value = time_element.xpath('text()')[0].split("：")
            if time_name == '下单':
                time_dict['order_time'] = time_value
            elif time_name == '付款':
                time_dict['pay_time'] = time_value
            elif time_name == '发货':
                time_dict['ship_time'] = time_value
            elif time_name == '提交':
                time_dict['submit_time'] = time_value
            else:
                time_dict[time_name] = time_value
    return [{
        'sku_list': sku_list,
        'currency': currency,
        'price': float(price.replace(',', '')),
        'recipient': recipient,
        'country': country,
        'order_id': order_id,
        'remark': remark,
        'time': time_dict
    }]


def get_merge_order_item_list(order_element_list: list[html.HtmlElement]):
    """
    解析获取合并单的订单项列表
    :return:
    """
    order_item_list = list()
    for order_element in order_element_list:
        order_item_list.extend(get_single_order_item_list(order_element))
    return order_item_list


def get_order_buyer_select_provider(good_element: html.HtmlElement):
    """
    获取买家指定
    :return:
    """
    # 第一种情况
    buyer_select_provider_list = good_element.xpath('.//span[contains(@class, "buyerSelectProvider")]/text()')
    # 第二种情况
    buyer_select_provider_2_list = good_element.xpath('./td[3]/text()')[0].strip().split("：")
    if len(buyer_select_provider_list) > 0:
        buyer_select_provider = buyer_select_provider_list[0]
    elif len(buyer_select_provider_2_list) == 2:
        buyer_select_provider = buyer_select_provider_2_list[1].strip()
    else:
        buyer_select_provider = None
    return buyer_select_provider


def list_order_base_by_html(html_str: str) -> list[dict]:
    """
    获取基本的订单分页信息
    有三种情况:
        1. 合并单
        2. 一单多件
        3. 普通一单一件
    :param html_str:
    :return:
    """
    tree = html.fromstring(html_str)
    good_element_list = tree.xpath('//tbody[@class="xianshishujudate"]/tr[@class="goodsId"]')
    order_list = list()
    table_columns = tree.xpath('//table[@id="orderListTable"]/thead/tr/th/text()')
    # 是否又物流方式
    has_logistics = True if '物流方式' in table_columns else False
    for good_element in good_element_list:
        # 包裹号
        package_number = good_element.xpath('.//a[@class="limingcentUrlpic"]/text()')[0]
        # 包裹id
        package_id = good_element.xpath('.//input[@class="input1"]/@value')[0]
        # authid 物流方式的id
        auth_id = good_element.xpath(f'.//input[@id="dxmAuthId{package_id}"]/@value')[0]
        # data-custom-mark标记
        data_custom_mark = get_str_list_first_not_blank_or_none(good_element.xpath('@data-custom-mark'))
        buyer_select_provider = get_order_buyer_select_provider(good_element)
        order_form_source = good_element.xpath('.//span[contains(@class, "order-form-source")]/text()')[0]
        platform, shop = order_form_source.replace('「', '').replace('」', '').split('：')
        order_class = f"orderId_{package_id}"
        order_element_list = tree.xpath(f'//tr[@class="{order_class}"]')
        order_first_element = order_element_list[0]
        # 订单备注
        hover_prompt_content_list = good_element.xpath('.//span[contains(@class, "hoverPrompt")]/@data-content')
        # 物流方式, 要先检查是否包含物流方式这一列
        logistics_info = None
        if has_logistics:
            # 物流方式
            logistics_list = order_first_element.xpath('.//td[6]/span/a/text()')
            # 可能又不存在的物流方式情况发生
            if len(logistics_list) > 0:
                logistics_info = dict()
                logistics = logistics_list[0].strip()
                # 物流单号
                logistics_info['track_number'] = get_str_list_first_not_blank_or_none(
                    order_first_element.xpath('.//td[6]//span[contains(@class, "limingcentUrlpicson")]/a/text()'))
                logistics_info['logistics'] = logistics
        # 状态
        if has_logistics:
            status = get_str_list_first_not_blank_or_none(order_first_element.xpath('.//td[7]/text()'))
        else:
            status = get_str_list_first_not_blank_or_none(order_first_element.xpath('.//td[6]/text()'))
        # 整理订单
        if len(order_element_list) > 1:
            # 合并单
            order_item_list = get_merge_order_item_list(order_element_list)
        else:
            # 普通单
            order_item_list = get_single_order_item_list(order_element_list[0])
        order_list.append({
            'package_id': package_id,
            'auth_id': auth_id,
            'package_number': package_number,
            'data_custom_mark': data_custom_mark,
            'buyer_select_provider': buyer_select_provider,
            'platform': platform,
            'shop': shop,
            'hover_prompt_content_list': hover_prompt_content_list,
            'logistics_info': logistics_info,
            'status': status,
            'order_item_list': order_item_list
        })
    return order_list


def list_order_rule(html_str: str):
    """
    获取订单规则列表
    :return:
    """
    tree = html.fromstring(html_str)
    rule_element_list = tree.xpath('//tbody[@id="ruleTbody"]/tr[@class="content rulesTr"]')
    rule_list = list()
    for rule_element in rule_element_list:
        rule_id = rule_element.xpath('./td[1]/input/@data-id')[0]
        rule_name = rule_element.xpath('./td[2]/text()')[0]
        status = rule_element.xpath('./td[5]/span/text()')[0]
        rule_list.append({
            'rule_id': rule_id,
            'rule_name': rule_name,
            'status': status
        })
    return rule_list


def get_rule_detail(html_str: str) -> DxmOrderRule:
    """
    根据订单规则的html获取订单规则详情
    :param html_str:
    :return:
    """
    tree = html.fromstring(html_str)
    cond_element_list = tree.xpath('//ul[@id="chooseSelectUl"]/li')
    cond_list = list()
    # 获取规则的条件
    for cond_element in cond_element_list:
        cond_val = cond_element.xpath('./input[@name="condVal"]/@value')[0]
        cond_id = cond_element.xpath('./input[@name="condId"]/@value')[0]
        cond_name = cond_element.xpath('./input[@name="condName"]/@value')[0]
        cond_unit = cond_element.xpath('./input[@name="condUnit"]/@value')[0]
        cond_type = cond_element.xpath('./input[@name="condType"]/@value')[0]
        cond_list.append(DxmOrderRuleCond(cond_val=cond_val,
                                          cond_id=cond_id,
                                          cond_name=cond_name,
                                          cond_unit=cond_unit,
                                          cond_type=cond_type))
    # 判断是物流规则还是审核规则
    modal_title = tree.xpath('//h4[@class="modal-title"]/text()')[0]
    rule_id = tree.xpath('//input[@name="id"]/@value')[0]
    rule_type = tree.xpath('//input[@name="type"]/@value')[0]
    rule_name = tree.xpath('//input[@id="ruleName"]/@value')[0]

    kfbz = get_str_list_first_not_blank_or_none(tree.xpath('//textarea[@name="kfbz"]/text()'))
    jhbz = get_str_list_first_not_blank_or_none(tree.xpath('//textarea[@name="jhbz"]/@value'))
    jh_color = tree.xpath('//a[contains(@class, "bgCommentFocus")]/@data-val')[0]
    other_action_checked_list = tree.xpath('//input[@name="otherAction"]/@checked')
    other_action = 'on' if len(other_action_checked_list) > 0 else ''
    custom_mark = tree.xpath('//div[contains(@class, "customMarkRuleEl")]/@data-custom-mark')[0]

    if modal_title.count('物流') > 0:
        warehouse_element_list = tree.xpath('//select[@id="ruleWareIdSelect"]//option')
        warehouse_id = ''
        distribute_type = ''
        for warehouse_element in warehouse_element_list:
            if warehouse_element.get('selected') == 'selected':
                warehouse_id = warehouse_element.get('value')
        distribute_type_element_list = tree.xpath('//input[@name="distributeType"]')
        for distribute_type_element in distribute_type_element_list:
            if distribute_type_element.get('checked') == 'checked':
                distribute_type = distribute_type_element.get('value')
        auth_ids = tree.xpath('//input[@name="authIds"]/@value')[0]
        pattern = re.compile(r"if\s*\(\s*'(\d+)'\s*===\s*item\.idStr")
        auth_id = pattern.findall(html_str)[0]

        # 物流规则
        dxm_rule_order = DxmOrderLogisticsRule(dxm_cond_list=cond_list, id=rule_id, type=rule_type,
                                               rule_name=rule_name, kfbz=kfbz, jhbz=jhbz,
                                               jh_color=jh_color, other_action=other_action,
                                               custom_mark=get_data_custom_mark_to_str(custom_mark),
                                               ware_id=warehouse_id, auth_id=auth_id, distribute_type=distribute_type,
                                               auth_ids=auth_ids
                                               )
    else:
        action = tree.xpath('//input[@name="action"]/@value')[0]
        dxm_rule_order = DxmOrderReviewRule(dxm_cond_list=cond_list, id=rule_id, type=rule_type,
                                            rule_name=rule_name, kfbz=kfbz, jhbz=jhbz,
                                            jh_color=jh_color, other_action=other_action,
                                            custom_mark=get_data_custom_mark_to_str(custom_mark),
                                            action=action
                                            )

    return dxm_rule_order


def get_order_detail_by_html(html_str: str):
    tree = html.fromstring(html_str)
    recipient = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailContact1"]/@data-info'))
    phone = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailPhone1"]/@data-info'))
    mobile = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailMobile1"]/@data-info'))
    country = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailCountry1"]/@data-country'))
    province = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailProvince1"]/text()'))
    city = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailCity1"]/text()'))
    addr1 = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailAddr11"]/@data-info'))
    addr2 = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailAddress21"]/@data-info'))
    company = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="companyName1"]/text()'))
    apartment = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="apartmentNumber1"]/text()'))
    zip_code = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="detailZip1"]/text()'))
    tax_number = get_str_list_first_not_blank_or_none(tree.xpath('//div[@id="taxNumber1"]/text()'))

    pair_info_element_list = tree.xpath('//tr[@class="orderInfoCon pairProInfoBox"]')
    ids = re.findall(r"var\s+storageId\s*=\s*['\"](\d+)['\"]", html_str)
    # 有就取第一个，否则 None
    storage_id = ids[0] if ids else None
    pair_info_list = []
    for pair_info_element in pair_info_element_list:
        pair_info_sku = pair_info_element.xpath('.//span[@class="pairProInfoSku"]/text()')[0].split(' x')[0].strip()
        pair_info_sku_quantity = int(pair_info_element.xpath('.//span[@class="pairProInfoSku"]/span/text()')[0].strip())
        # warehouse_sku, warehouse_sku_quantity = pair_info_element.xpath(
        #     './/div[contains(@class, "normalDiv")]/p[1]/text()')[0].split(' x ')
        warehouse_sku_info_list = pair_info_element.xpath('.//div[contains(@class, "normalDiv")]/p[1]/text()')
        warehouse_sku, warehouse_sku_quantity = None, None
        if len(warehouse_sku_info_list) > 0:
            warehouse_sku_info = warehouse_sku_info_list[0]
            warehouse_sku, warehouse_sku_quantity = warehouse_sku_info.split(' x ')

        warehouse_available_quantity_element_list = pair_info_element.xpath(
            './/div[contains(@class, "normalDiv")]/p[2]/span[2]/text()')
        warehouse_available_quantity = int(warehouse_available_quantity_element_list[0].strip()) if len(
            warehouse_available_quantity_element_list) > 0 else None
        pair_info_list.append({
            'pair_info_sku': pair_info_sku,
            'pair_info_sku_quantity': pair_info_sku_quantity,
            'warehouse_sku': warehouse_sku,
            'warehouse_sku_quantity': warehouse_sku_quantity,
            'warehouse_available_quantity': warehouse_available_quantity
        })
    return {
        'recipient': recipient,
        'phone': phone,
        'mobile': mobile,
        'country': country,
        'province': province,
        'city': city,
        'addr1': addr1,
        'addr2': addr2,
        'company': company,
        'apartment': apartment,
        'zip_code': zip_code,
        'tax_number': tax_number,
        'pair_info_list': pair_info_list,
        'storage_id': storage_id
    }



from typing import TypedDict, Optional
from urllib.parse import urlencode

from ey_commerce_lib.utils.str import to_camel
from pydantic import BaseModel, ConfigDict


class DxmOrderSearchForm(BaseModel):
    """
    /package/advancedSearch.htm
    店小秘订单搜索接口参数约束
    """
    # 搜索类型(orderId订单号, packageNum包裹号)
    search_types: str = 'orderId'
    # 搜索内容
    contents: str = ''
    order_adv_search_type: str = '1'
    # 订单状态
    state: str = ''
    is_voided: str = '-1'
    is_removed: str = '-1'
    commit_platforms: str = ''
    # 是否为海外仓-1代表全部,1是0否
    is_oversea: str = '-1'
    # 店铺id
    shop_id: str = '-1'
    # 平台
    platform: str = ''
    # 历史订单(空字符串就是代表没有, all 代表历史记录)
    history: str = ''
    # 排序方式
    order_field: str = 'order_create_time'
    is_desc: str = '1'
    time_out: str = '0'
    warehouse_code: str = ''
    # 订单标记绿色
    is_green: str = '0'
    # 订单标记黄色
    is_yellow: str = '0'
    # 订单标记橙色
    is_orange: str = '0'
    # 订单标记红色
    is_red: str = '0'
    # 订单标记紫色
    is_violet: str = '0'
    # 订单标记蓝色
    is_blue: str = '0'
    # 订单标记青色
    corn_flower_blue: str = '0'
    # 订单标记粉色
    pink: str = '0'
    teal: str = '0'
    turquoise: str = '0'
    unmarked: str = '0'
    forbidden_status: str = '-1'
    forbidden_reason: str = '0'
    picking_instructions: str = ''
    # 订单金额起始
    price_start: str = ''
    # 订单金额结束
    price_end: str = ''
    # 下单时间
    order_create_start: str = ''
    order_create_end: str = ''
    # 付款时间
    order_pay_start: str = ''
    order_pay_end: str = ''
    # 发货时间
    shipped_start: str = ''
    shipped_end: str = ''
    # 退款时间
    refunded_start: str = ''
    refunded_end: str = ''
    # 面单打印时间
    md_sign_start: str = ''
    md_sign_end: str = ''
    # 拣货单打印时间
    jh_sign_start: str = ''
    jh_sign_end: str = ''
    # 剩余发货
    time_out_query: str = '-1'
    # 包裹类型
    product_status: str = ''
    # 订单商品数量
    product_count_start: str = ''
    product_count_end: str = ''
    # 发货仓库, 多个仓库用逗号隔开
    storage_ids: str = ''
    storage_id: str = '0'
    auth_id: str = '-1'
    days: str = '-1'
    # 国家区域
    country: str = ''
    # 拣货单打印（-1全部,1已打拣货单,2未打拣货单）
    is_print_jh: str = '-1'
    # 优先分配库存(-1全部,1是,2否)
    sign_prior_ship: str = '-1'
    # 面单打印(-1全部,1是,2否)
    is_print_md: str = '-1'
    # 提交单号(uncommit)
    commit_platform: str = ''
    # 买家留言(-1全部,1有留言,0没有留言)
    is_has_order_message: str = '-1'
    # 买家备注(-1全部,1有备注,0没有备注)
    is_has_order_comment: str = '-1'
    # 客服备注(-1全部,1有备注,0没有备注)
    is_has_service_comment: str = '-1'
    # 拣货备注(-1全部,1有备注,0没有备注)
    is_has_pick_comment: str = '-1'

    # 合并订单(1代表勾选)
    is_merge: str = ''
    # 拆分订单(1代表勾选)
    is_split: str = ''
    # 重发包裹(1代表勾选)
    is_re_ship: str = ''
    # 有退款
    is_refund: str = ''
    # 禁止处理(1代表勾选)
    sign_ban_ship: str = ''
    # 黑名单
    black_list: str = ''
    # 平台标识
    global_collection: str = '-1'
    # 分页
    page_no: int = 1
    page_size: int = 30

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True  # 对应旧版的 allow_population_by_field_name
    )


class DxmJsonResponse(TypedDict):
    """
    店小秘一般json返回响应
    """
    code: int
    msg: str


class DxmProcessMsg(TypedDict):
    code: int
    msg: str
    data: Optional[dict]
    num: int
    totalNum: int
    createDate: int


class DxmCheckProcessResponse(TypedDict):
    """
    店小秘检测进度返回参数类型
    """
    processMsg: DxmProcessMsg


class DxmOrderRuleCond(BaseModel):
    """
    店小秘订单规则参数项基类
    """
    # 条件值
    cond_val: str
    # 条件id
    cond_id: str
    # 条件名称
    cond_name: str
    # 条件单位
    cond_unit: str = ''
    # 条件类型
    cond_type: str = '1'


class DxmOrderRule(BaseModel):
    """
    店小秘订单规则参数基类
    """
    # 规则id
    id: str
    # 规则类型(1:审单规则，2:物流规则)
    type: str
    # 规则名称
    rule_name: str
    # 客服备注
    kfbz: str = ''
    # 拣货备足
    jhbz: str = ''
    # 拣货颜色
    jh_color: str = ''
    # 自定义备注
    custom_mark: str = ''
    # 附加规则(on勾选, ''未勾选)
    other_action: str = ''
    has_other_action: str = ''
    # 条件列表
    dxm_cond_list: list[DxmOrderRuleCond]

    def to_update_rule_data_before(self):
        data = list()
        data.append(('id', self.id))
        data.append(('type', self.type))
        data.append(('ruleName', self.rule_name))
        # 装填规则内容
        for cond in self.dxm_cond_list:
            data.append(('condVal', cond.cond_val))
            data.append(('condId', cond.cond_id))
            data.append(('condName', cond.cond_name))
            data.append(('condUnit', cond.cond_unit))
            data.append(('condType', cond.cond_type))
        # 装填conditionId
        for cond in self.dxm_cond_list:
            data.append(('conditionId', cond.cond_id))
        return data

    def to_update_rule_data_after(self):
        data = list()
        data.append(('otherAction', self.other_action))
        data.append(('kfbz', self.kfbz))
        data.append(('jhbz', self.jhbz))
        data.append(('hasOtherAction', self.has_other_action))
        data.append(('customMark', self.custom_mark))
        data.append(('jhColor', self.jh_color))
        return data

    def to_update_rule_data(self):
        """
        转换成更新物流规则的参数数据,由子类实现
        :return:
        """
        pass


class DxmOrderReviewRule(DxmOrderRule):
    """
    店小秘审单规则参数
    """
    # 流程规则(paid: 待审核, approved: 待处理)
    action: str = 'paid'

    def to_update_rule_data(self):
        """
        转换成更新物流规则的参数数据
        :return:
        """
        data = super().to_update_rule_data_before()
        # 装填其他参数
        data.append(('action', self.action))
        data.extend(super().to_update_rule_data_after())
        return urlencode(data, encoding="utf-8")


class DxmOrderLogisticsRule(DxmOrderRule):
    """
    店小秘订单物流规则参数
    """
    ware_id: str
    # 物流方式id(物流规则使用, 必须要是distribute_type为0)
    auth_id: str
    # 分配方式(0:仅指定一个物流,1:指定多个物流)
    distribute_type: str
    # 物流方式ids(物流规则使用, 多个用逗号隔开, 必须要是distribute_type为1)
    auth_ids: str

    def to_update_rule_data(self):
        data = super().to_update_rule_data_before()
        # 装填其他参数
        data.append(('wareId', self.ware_id))
        data.append(('distributeType', self.distribute_type))
        data.append(('authId', self.auth_id))
        data.append(('authIds', self.auth_ids))
        data.extend(super().to_update_rule_data_after())
        return urlencode(data, encoding="utf-8")



from typing import Optional
from pydantic import BaseModel, Field, ConfigDict


class FourSellerOrderAddressVo(BaseModel):
    # 地址 ID
    id: int

    # 收件人姓名
    name: str

    # 城市
    city: str

    # 邮编
    postal_code: Optional[str] = Field(..., alias="postalCode")

    # 完整地址
    address: str

    # 电话
    phone: str

    # 邮箱
    email: str

    # 国家代码
    country_code: str = Field(..., alias="countryCode")

    # 区/县
    county: Optional[str]

    # 地址行1
    address1: str

    # 地址行2
    address2: Optional[str]

    # 地址行3
    address3: Optional[str] = Field(None)

    # 州/省
    state: Optional[str] = Field(None)

    # 地址备注
    msg: Optional[str] = Field(None)

    # 地址备注映射
    msg_map: Optional[dict] = Field(None, alias="msgMap")

    # 是否住宅
    residential: Optional[bool]

    # 是否检查地址
    check_address: bool = Field(..., alias="checkAddress")

    model_config = ConfigDict(
        populate_by_name=True
    )


class FourSellerTimeInfoVO(BaseModel):
    # 订单支付时间
    order_paid_time: Optional[str] = Field(..., alias="orderPaidTime")

    # 订单创建时间
    order_create_time: str = Field(..., alias="orderCreateTime")

    # 最晚发货日期
    latest_ship_date: Optional[str] = Field(..., alias="latestShipDate")

    # 发货时间
    order_shipped_time: Optional[str] = Field(..., alias="orderShippedTime")

    # 最晚送达日期
    latest_delivery_date: Optional[str] = Field(..., alias="latestDeliveryDate")

    # 取消时间
    cancel_time: Optional[str] = Field(..., alias="cancelTime")

    # 剩余天数
    leave_day: int = Field(..., alias="leaveDay")

    model_config = ConfigDict(
        populate_by_name=True
    )


class FourSellerPackageInfoVO(BaseModel):
    # 重量
    weight: Optional[float]

    # 重量单位
    weight_unit: Optional[str] = Field(None, alias="weightUnit")

    # 包裹 ID
    package_id: Optional[int] = Field(None, alias="packageId")

    # 包裹类型
    package_type: Optional[str] = Field(None, alias="packageType")

    # 包裹名称
    package_name: Optional[str] = Field(None, alias="packageName")

    # 平台包裹类型
    platform_package_type: Optional[str] = Field(None, alias="platformPackageType")

    # 长度
    length: Optional[float]

    # 宽度
    width: Optional[float]

    # 高度
    height: Optional[float]

    # 尺寸单位
    dimension_unit: Optional[str] = Field(None, alias="dimensionUnit")

    model_config = ConfigDict(
        populate_by_name=True
    )


class FourSellerShippingServiceInfoVO(BaseModel):
    # 保险金额
    insurance_amount: Optional[float] = Field(None, alias="insuranceAmount")

    # 保险币种
    insurance_currency: Optional[str] = Field(None, alias="insuranceCurrency")

    # 物流服务
    shipping_service: Optional[str] = Field(None, alias="shippingService")

    # 承运商
    carrier: Optional[str]

    # 运费
    shipping_fee: Optional[float] = Field(None, alias="shippingFee")

    # 币种
    currency: Optional[str]

    # 预估送达天数
    estimated_delivery_days: Optional[int] = Field(None, alias="estimatedDeliveryDays")

    # 物流授权 ID
    logistics_auth_id: Optional[int] = Field(None, alias="logisticsAuthId")

    # 物流承运商 ID
    logistics_carrier_id: Optional[int] = Field(None, alias="logisticsCarrierId")

    # 物流承运商代码
    logistics_carrier_code: Optional[str] = Field(None, alias="logisticsCarrierCode")

    # 物流承运商名称
    logistics_carrier_name: Optional[str] = Field(None, alias="logisticsCarrierName")

    # 物流平台
    logistics_platform: Optional[str] = Field(None, alias="logisticsPlatform")

    # 物流服务 ID
    logistics_service_id: Optional[int] = Field(None, alias="logisticsServiceId")

    # 物流服务代码
    logistics_service_code: Optional[str] = Field(None, alias="logisticsServiceCode")

    # 物流服务类型
    logistics_service_type: Optional[str] = Field(None, alias="logisticsServiceType")

    # 物流服务名称
    logistics_service_name: Optional[str] = Field(None, alias="logisticsServiceName")

    # 平台物流服务 ID
    platform_shipping_service_id: Optional[int] = Field(None, alias="platformShippingServiceId")

    # 空白箱
    blank_box: Optional[int] = Field(None, alias="blankBox")

    # 阻止 AMZL
    block_amzl: Optional[int] = Field(None, alias="blockAmzl")

    # ERP 预估发货日期
    erp_estimated_shipping_date: Optional[str] = Field(None, alias="erpEstimatedShippingDate")

    # 配送确认类型
    delivery_confirmation_type: Optional[str] = Field(None, alias="deliveryConfirmationType")

    # 标签价格
    label_price: Optional[float] = Field(None, alias="labelPrice")

    # 预估运费
    estimated_shipping_price: Optional[float] = Field(None, alias="estimatedShippingPrice")

    model_config = ConfigDict(
        populate_by_name=True
    )


class FourSellerOrderItemInfoVO(BaseModel):
    # 订单商品 ID
    order_item_id: int = Field(..., alias="orderItemId")

    # 订单 ID
    order_id: int = Field(..., alias="orderId")

    # 卖家 SKU
    seller_sku: Optional[str] = Field(..., alias="sellerSku")

    # 商品标题
    title: str

    # 图片 URL
    img_url: Optional[str] = Field(None, alias="imgUrl")

    # 列表 URL
    listing_url: Optional[str] = Field(None, alias="listingUrl")

    # SKU ID
    sku_id: Optional[int] = Field(..., alias="skuId")

    # SKU
    sku: Optional[str] = Field(None)

    # 数量
    quantity: int

    # 变体属性
    variant_attr: Optional[str] = Field(None, alias="variantAttr")

    # 单价
    item_price: Optional[float] = Field(..., alias="itemPrice")

    # 币种
    currency: Optional[str] = Field(None)

    # 是否占用库存
    occupy: Optional[bool] = Field(None)

    # 商品是否未找到
    product_not_found: bool = Field(..., alias="productNotFound")

    # 地址是否有效
    address_valid: bool = Field(..., alias="addressValid")

    # 地址错误信息
    address_error_msg: Optional[str] = Field(None, alias="addressErrorMsg")

    # 来源 URL
    source_url: Optional[str] = Field(None, alias="sourceUrl")

    # 是否预售
    pre_sale: int = Field(..., alias="preSale")

    model_config = ConfigDict(
        populate_by_name=True
    )


class FourSellerOrderVO(BaseModel):
    # 订单 ID
    order_id: int = Field(..., alias="orderId")

    # 金额
    amount: Optional[float] = Field(None)

    # 币种
    currency: Optional[str] = Field(None)

    # 仓库 ID
    warehouse_id: int = Field(..., alias="warehouseId")

    # 仓库类型
    warehouse_type: str = Field(..., alias="warehouseType")

    # 仓库代码
    warehouse_code: Optional[str] = Field(None, alias="warehouseCode")

    # 平台仓名称
    platform_warehouse_name: Optional[str] = Field(None, alias="platformWarehouseName")

    # 平台物流方式
    platform_shipment_method: Optional[str] = Field(None, alias="platformShipmentMethod")

    # 支付状态
    paid_status: Optional[str] = Field(None, alias="paidStatus")

    # 订单状态
    order_status: str = Field(..., alias="orderStatus")

    # 平台订单状态
    platform_order_status: str = Field(..., alias="platformOrderStatus")

    # 取消原因
    cancel_reason: Optional[str] = Field(None, alias="cancelReason")

    # 平台
    platform: str

    # 平台订单号
    platform_order_id: str = Field(..., alias="platformOrderId")

    # 平台订单编号
    platform_order_no: str = Field(..., alias="platformOrderNo")

    # 店铺 ID
    shop_id: int = Field(..., alias="shopId")

    # 店铺名称
    shop_name: Optional[str] = Field(None, alias="shopName")

    # 包裹名称
    package_name: Optional[str] = Field(None, alias="packageName")

    # 买家备注
    buyer_memo: Optional[str] = Field(None, alias="buyerMemo")

    # 卖家备注
    seller_memo: Optional[str] = Field(None, alias="sellerMemo")

    # 内部备注
    internal_memo: Optional[str] = Field(None, alias="internalMemo")

    # 发货单号
    shipment_no: Optional[str] = Field(None, alias="shipmentNo")

    # 是否三方仓
    third_party: bool = Field(..., alias="thirdParty")

    # 是否打印标签
    print_label: bool = Field(..., alias="printLabel")

    # 是否打印装箱单
    print_packing_slip: bool = Field(..., alias="printPackingSlip")

    # 是否打印拣货单
    print_pick: bool = Field(..., alias="printPick")

    # 配送选项
    delivery_option_type: Optional[int] = Field(None, alias="deliveryOptionType")

    # 买标状态
    buy_label_process_status: str = Field(..., alias="buyLabelProcessStatus")

    # 错误信息
    error_message: Optional[str] = Field(None, alias="errorMessage")

    # 发货类型
    shipment_type: Optional[str] = Field(..., alias="shipmentType")

    # 用户设置发货类型
    user_setting_shipment_type: str = Field(..., alias="userSettingShipmentType")

    # 发货地址 ID
    ship_from_address_id: Optional[int] = Field(None, alias="shipFromAddressId")

    # 是否可拆分
    can_split: Optional[bool] = Field(..., alias="canSplit")

    # 追踪号
    tracking_no: Optional[str] = Field(None, alias="trackingNo")

    # 物流方式
    shipping_method: str = Field(..., alias="shippingMethod")

    # 标签列表
    tag_list: Optional[list[str]] = Field(None, alias="tagList")

    # 新标签列表
    new_tag_list: Optional[list[dict]] = Field(None, alias="newTagList")

    # 拆单编号
    split_no: int = Field(..., alias="splitNo")

    # 是否拆单
    is_split: int = Field(..., alias="isSplit")

    # ERP 是否标记发货
    is_erp_mark_ship: int = Field(..., alias="isErpMarkShip")

    # 标签地址
    shipping_label_url: Optional[str] = Field(None, alias="shippingLabelUrl")

    # 配送渠道类型
    fulfillment_channel_type: int = Field(..., alias="fulfillmentChannelType")

    # 标签打印时间
    print_label_time: Optional[str] = Field(None, alias="printLabelTime")

    # 拣货单打印时间
    print_pick_time: Optional[str] = Field(None, alias="printPickTime")

    # 装箱单打印时间
    print_packing_slip_time: Optional[str] = Field(None, alias="printPackingSlipTime")

    # 是否 17 物流挂号
    is_register17: int = Field(..., alias="isRegister17")

    # 第三方仓状态
    third_party_warehouse_status: Optional[str] = Field(None, alias="thirdPartyWarehouseStatus")

    # 订单类型
    order_type: int = Field(..., alias="orderType")

    # 被合并订单列表
    be_merged_order_list: Optional[list[dict]] = Field(None, alias="beMergedOrderList")

    # 买家申请取消
    is_buyer_requested_cancel: int = Field(..., alias="isBuyerRequestedCancel")

    # 是否作废
    is_voided: int = Field(..., alias="isVoided")

    # 地址信息
    order_address_vo: Optional[FourSellerOrderAddressVo] = Field(..., alias="orderAddressVo")

    # 时间信息
    time_info: FourSellerTimeInfoVO = Field(..., alias="timeInfo")

    # 包裹信息
    package_info: FourSellerPackageInfoVO = Field(..., alias="packageInfo")

    # 物流服务信息
    shipping_service_info: FourSellerShippingServiceInfoVO = Field(..., alias="shippingServiceInfo")

    # 商品列表
    order_item_info_list: list[FourSellerOrderItemInfoVO] = Field(..., alias="orderItemInfoList")

    # 是否样品订单
    is_sample_order: int = Field(..., alias="isSampleOrder")

    # 是否预售
    pre_sale: Optional[int] = Field(..., alias="preSale")

    model_config = ConfigDict(
        populate_by_name=True
    )



from typing import Optional

from ey_commerce_lib.utils.str import to_camel
from pydantic import BaseModel, ConfigDict


class WarehouseProductQuery(BaseModel):
    refresh_flag: str = ''
    # 8(全部区)6(次品区)0(拣货区)
    zone_type: int = 0
    # 页码
    page_no: int = 1
    # 每页条数
    page_size: int = 100
    # 搜索类型1(商品sku)4(商品编码)2(商品名称)3(货架位)4(识别码)
    search_type: int = 1
    # 搜索内容
    search_value: str = ''
    # 匹配方式1(精确匹配)0(模糊匹配)2(完全匹配)
    product_search_type: int = 1
    # 仓库id
    warehouse_id: str = '7167025'
    is_transit: str = ''
    # 筛选排序 1(按照创建时间)2(按更新时间)3(按sku)4(按库存总量)5(按单价)6(按采购在途)7(按总价)8(按货架位)9(按可用库存)10(按未发)
    order_by: str = '1'
    # 排序方式 1(降序)0(升序)
    order_by_val: str = '1'
    # 分类路径用-分隔最后以-结尾 例如1379550-1379554-1379559-
    full_cid: str = ''
    # 类型 全部('')单个sku(0)组合sku(1)加工sku(2)
    group_or_not: str = ''
    # 商品单价开始
    price_min: str = ''
    # 商品单价结束
    price_max: str = ''
    # 总库存起始
    stock_min: str = ''
    # 总库存结束
    stock_max: str = ''
    # 可用库存起始
    available_min: str = ''
    # 可用库存结束
    available_max: str = ''
    # 安全库存开始
    safe_min: str = ''
    # 安全库存结束
    safe_max: str = ''
    # 采购在途开始
    on_pass_min: str = ''
    # 采购在途结束
    on_pass_max: str = ''
    # 预售数量起始
    lock_min: str = ''
    # 预售数量结束
    lock_max: str = ''
    # 未发数量起始
    un_billed_order_min: str = ''
    # 未发数量结束
    un_billed_order_max: str = ''
    # 仓库商品状态 -1(全部)0(在售)1(热销)2(新品)3(清仓)4(停售)5(滞销)
    product_status: str = '-1'

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True  # 对应旧版的 allow_population_by_field_name
    )


class WarehouseProduct(BaseModel):
    sku: str
    name: Optional[str] = None
    sku_code: str
    shelf_position: str
    # 安全库存
    safe_stock: Optional[int] = None
    # 在途库存
    on_the_way_stock: Optional[int] = None
    # 未发库存
    not_shipped_stock: Optional[int] = None
    # 占用库存
    occupy_stock: Optional[int] = None
    # 可用库存
    available_stock: Optional[int] = None
    # 总库存
    total_stock: Optional[int] = None
    # 更新时间
    update_time: str
    # 创建时间
    create_time: str


class PurchasingAllQuery(BaseModel):
    """
    采购订单全部查询参数
    """
    page_no: int = 1
    page_size: int = 300
    # 搜索类型0(采购单号)1(供货商)2(商品sku)6(商品编码)5(商品名称)4(1688单号)3(运单号)7(关联单号)8(采购单备注)9(商品备注)
    search_type: int = 0
    # 搜索内容
    search_value: str = ''
    # 搜索模式1(精确匹配)0(模糊匹配)2(完全匹配)
    search_mode: int = 1
    # 评论颜色使用,分隔
    comment_colors: str = ''
    # 仓库id
    warehouse_id: str = ''

    # 店铺id(或者1688账号)
    authorization_id: str = ''
    # 排序方式: 1(按创建时间) 2(按更新时间) 4(按照采购单号)5(按照供货商名)
    purchasing_px: int = 2
    # 排序方式: asc(升序) desc(降序)
    purchasing_px_val: str = 'desc'
    is_advance_search: str = 'Y'
    # 采购金额起始
    price_left: str = ''
    # 采购金额结束
    price_right: str = ''
    # 采购数量起始
    quantity_left: str = ''
    # 采购数量结束
    quantity_right: str = ''
    # 创建时间开始
    create_start_time: str = ''
    # 创建时间结束
    create_end_time: str = ''
    # 快递签收时间开始
    end_time_start: str = ''
    # 快递签收时间结束
    end_time_end: str = ''
    # 到货时间开始
    arrive_start_time: str = ''
    # 到货时间结束
    arrive_end_time: str = ''
    # 供货商id
    supplier_id: str = ''
    # 采购员id
    agent_id: str = ''
    # 结算方式 1(款到发货)2(货到付款)3(账期付款)4(其它)5(账期付款周结)6(账期付款半月结)7(账期付款月结)
    settlement_method: str = ''
    # 订单状态
    cur_states: str = ''
    # 付款状态
    submit_states: str = ''
    # 到货状态
    arrive_state: str = ''
    # 消息标记-有留言(勾选是Y)
    has_comment: str = ''
    # 消息标记-有备注(勾选是Y)
    has_remark: str = ''
    # 无备注
    no_has_comment: str = ''
    # 下单类型2(1688采购)3(手工采购)''(全部)
    is_alibaba: str = ''
    # 物流信息 ''(全部)2(有运单号)3(无运单号)
    is_have_identify_no: str = ''

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True
    )






import math
from typing import TypedDict


class RemainingShipTime(TypedDict):
    day: int
    hour: int
    minute: int
    second: int


def get_remaining_ship_time(time_int) -> RemainingShipTime:
    """
    获取剩余发货时间
    :return:
    """
    d = math.floor(time_int / 86400)
    h = math.floor((time_int % 86400) / 3600)
    m = math.floor(((time_int % 86400) % 3600) / 60)
    s = math.floor(((time_int % 86400) % 3600) % 60)
    return {
        "day": d,
        "hour": h,
        "minute": m,
        "second": s
    }


def get_data_custom_mark_to_str(custom_mark: str) -> str:
    mark_list = custom_mark.split(',')
    color_str_list = list()
    mark_color_list = ['isGreen', 'isYellow', 'isOrange', 'isRed', 'isViolet', 'isBlue', 'cornflowerBlue', 'pink',
                       'teal',
                       'turquoise']

    for index in range(len(mark_list)):
        if index >= len(mark_list):
            break
        # print(mark_list[index])
        if mark_list[index] == '0':
            continue
        color_str_list.append(f'{mark_color_list[index]}_{mark_list[index]}')
    return '&'.join(color_str_list)



from typing import Optional
from pydantic import BaseModel, Field, ConfigDict


class KoganProductQuery(BaseModel):
    enabled: Optional[bool] = Field(
        None,
        alias="enabled",
        description="启用状态过滤（True/true/1 表示启用，False/false/0 表示禁用）"
    )
    category: Optional[str] = Field(
        None,
        alias="category",
        description="按 Kogan.com 类别名称筛选（不区分大小写，可多选）"
    )
    brand: Optional[str] = Field(
        None,
        alias="brand",
        description="按品牌筛选（不区分大小写，可多选，标题中逗号需要转义，例如 '\\,'）"
    )
    created: Optional[str] = Field(
        None,
        alias="created",
        description="创建时间"
    )
    ebay_category: Optional[int] = Field(
        None,
        alias="ebay_category",
        description="按 eBay 类别 ID 筛选"
    )
    sku: Optional[str] = Field(
        None,
        alias="sku",
        description="按 SKU 精确匹配（支持多选）"
    )
    search: Optional[str] = Field(
        None,
        alias="search",
        description="搜索商品标题"
    )
    ordering: Optional[str] = Field(
        None,
        alias="ordering",
        description="结果排序字段"
    )
    cursor: Optional[str] = Field(
        None,
        alias="cursor",
        description="用于分页的游标（续接 'next' URL 的分页查询）"
    )
    size: Optional[int] = Field(
        None,
        alias="size",
        description="每页返回结果数量"
    )
    created_after: Optional[str] = Field(
        None,
        alias="created_after",
        description="筛选创建日期 >= 指定日期（AEST/AEDT 格式 YYYY-MM-DD）"
    )
    created_before: Optional[str] = Field(
        None,
        alias="created_before",
        description="筛选创建日期 <= 指定日期（AEST/AEDT 格式 YYYY-MM-DD）"
    )
    detail: Optional[bool] = Field(
        None,
        alias="detail",
        description="是否启用响应中所有字段（True/true/1 表示启用）"
    )

    model_config = ConfigDict(
        # 支持别名
        populate_by_name=True
    )



from pydantic import BaseModel, Field, ConfigDict
from typing import Optional


class OfferDetail(BaseModel):
    # 价格
    price: str = Field(..., alias="price")
    # Kogan First 价格
    kogan_first_price: Optional[str] = Field(None, alias="kogan_first_price")
    # 是否免税
    tax_exempt: bool = Field(..., alias="tax_exempt")
    # 运费
    shipping: str = Field(..., alias="shipping")
    # 处理天数
    handling_days: int = Field(..., alias="handling_days")
    # 建议零售价
    rrp: Optional[str] = Field(None, alias="rrp")

    model_config = ConfigDict(
        populate_by_name=True
    )


class StoreUrl(BaseModel):
    # 商店链接
    url: str = Field(..., alias="url")
    # 商店名称
    store_name: str = Field(..., alias="store_name")
    # 组织名称
    organisation: str = Field(..., alias="organisation")

    model_config = ConfigDict(
        populate_by_name=True
    )


class Product(BaseModel):
    # 产品标题
    product_title: str = Field(..., alias="product_title")
    # 产品SKU
    product_sku: str = Field(..., alias="product_sku")
    # 产品副标题
    product_subtitle: str = Field(..., alias="product_subtitle")
    # 产品 GTIN
    product_gtin: str = Field(..., alias="product_gtin")
    # 图片链接列表
    images: list[str] = Field(..., alias="images")
    # 品牌
    brand: Optional[str] = Field(None, alias="brand")
    # 类别
    category: str = Field(..., alias="category")
    # 类别标识
    category_slug: str = Field(..., alias="category_slug")
    # 报价数据
    offer_data: dict[str, OfferDetail] = Field(..., alias="offer_data")
    # 库存数量
    stock: int = Field(..., alias="stock")
    # 是否启用
    enabled: bool = Field(..., alias="enabled")
    # 创建时间
    created: str = Field(..., alias="created")
    # 商店地址列表
    store_urls: list[StoreUrl] = Field(..., alias="store_urls")
    # 标签
    tags: list[str] = Field(..., alias="tags")

    model_config = ConfigDict(
        populate_by_name=True
    )


class Body(BaseModel):
    # 下一页链接
    next: Optional[str] = Field(None, alias="next")
    # 产品列表
    results: list[Product] = Field(..., alias="results")

    model_config = ConfigDict(
        populate_by_name=True
    )


class ProductResponse(BaseModel):
    # 状态
    status: str = Field(..., alias="status")
    # 待处理URL
    pending_url: Optional[str] = Field(None, alias="pending_url")
    # 错误信息
    error: Optional[str] = Field(None, alias="error")
    # 响应主体
    body: Body = Field(..., alias="body")

    model_config = ConfigDict(
        populate_by_name=True
    )

